@@ -15,8 +15,9 @@
  */
 
 use super::wit_prelude::*;
+use super::MFunctionSignature;
+use super::MRecordTypes;
 use super::{IType, IRecordType, IFunctionArg, IValue, WValue};
-use super::RecordTypes;
 use crate::MResult;
 use crate::MModuleConfig;
 
@@ -29,9 +30,6 @@ use wasmer_runtime::compile;
 use wasmer_runtime::ImportObject;
 use wasmer_it::interpreter::Interpreter;
 
-use serde::Serialize;
-use serde::Deserialize;
-
 use std::collections::HashMap;
 use std::convert::TryInto;
 use std::mem::MaybeUninit;
@@ -48,18 +46,10 @@ pub(super) struct ITModuleFunc {
     pub(super) output_types: Rc<Vec<IType>>,
 }
 
-/// Represent a function type inside Marine module.
-#[derive(PartialEq, Eq, Debug, Clone, Hash, Serialize, Deserialize)]
-pub struct MFunctionSignature {
-    pub name: Rc<String>,
-    pub arguments: Rc<Vec<IFunctionArg>>,
-    pub outputs: Rc<Vec<IType>>,
-}
-
 #[derive(Clone)]
 pub(super) struct Callable {
-    pub(super) wit_instance: Arc<ITInstance>,
-    pub(super) wit_module_func: ITModuleFunc,
+    pub(super) it_instance: Arc<ITInstance>,
+    pub(super) it_module_func: ITModuleFunc,
 }
 
 impl Callable {
@@ -67,9 +57,9 @@ impl Callable {
         use wasmer_it::interpreter::stack::Stackable;
 
         let result = self
-            .wit_module_func
+            .it_module_func
             .interpreter
-            .run(args, Arc::make_mut(&mut self.wit_instance))?
+            .run(args, Arc::make_mut(&mut self.it_instance))?
             .as_slice()
             .to_owned();
 
@@ -105,7 +95,7 @@ pub(crate) struct MModule {
 
     // TODO: save refs instead copying of a record types HashMap.
     /// Record types used in exported functions as arguments or return values.
-    export_record_types: RecordTypes,
+    export_record_types: MRecordTypes,
 }
 
 impl MModule {
@@ -130,7 +120,7 @@ impl MModule {
             Self::create_import_objects(config, &mit, wit_import_object.clone())?;
 
         let wasmer_instance = wasmer_module.instantiate(&wasi_import_object)?;
-        let wit_instance = unsafe {
+        let it_instance = unsafe {
             // get_mut_unchecked here is safe because currently only this modules have reference to
             // it and the environment is single-threaded
             *Arc::get_mut_unchecked(&mut wit_instance) =
@@ -138,8 +128,7 @@ impl MModule {
             std::mem::transmute::<_, Arc<ITInstance>>(wit_instance)
         };
 
-        let export_funcs = Self::instantiate_wit_exports(&wit_instance, &mit)?;
-        let export_record_types = Self::extract_export_record_types(&export_funcs, &wit_instance)?;
+        let (export_funcs, export_record_types) = Self::instantiate_exports(&it_instance, &mit)?;
 
         // call _start to populate the WASI state of the module
         #[rustfmt::skip]
@@ -179,12 +168,12 @@ impl MModule {
             .iter()
             .map(|(func_name, func)| MFunctionSignature {
                 name: func_name.0.clone(),
-                arguments: func.wit_module_func.arguments.clone(),
-                outputs: func.wit_module_func.output_types.clone(),
+                arguments: func.it_module_func.arguments.clone(),
+                outputs: func.it_module_func.output_types.clone(),
             })
     }
 
-    pub(crate) fn export_record_types(&self) -> &RecordTypes {
+    pub(crate) fn export_record_types(&self) -> &MRecordTypes {
         &self.export_record_types
     }
 
@@ -260,54 +249,36 @@ impl MModule {
         Ok((wasi_import_object, host_closures_import_object))
     }
 
-    fn instantiate_wit_exports(
-        wit_instance: &Arc<ITInstance>,
-        wit: &MITInterfaces<'_>,
-    ) -> MResult<ExportFunctions> {
-        use marine_it_interfaces::ITAstType;
+    fn instantiate_exports(
+        it_instance: &Arc<ITInstance>,
+        mit: &MITInterfaces<'_>,
+    ) -> MResult<(ExportFunctions, MRecordTypes)> {
+        let module_interface = marine_module_interface::it_interface::get_interface(mit)?;
 
-        wit.implementations()
-            .filter_map(|(adapter_function_type, core_function_type)| {
-                wit.exports_by_type(*core_function_type)
-                    .map(|export_function_name| (adapter_function_type, export_function_name))
+        let export_funcs = module_interface
+            .function_signatures
+            .into_iter()
+            .map(|sign| {
+                let adapter_instructions = mit.adapter_by_type_r(sign.adapter_function_type)?;
+
+                let interpreter: ITInterpreter = adapter_instructions.clone().try_into()?;
+                let it_module_func = ITModuleFunc {
+                    interpreter: Arc::new(interpreter),
+                    arguments: sign.arguments.clone(),
+                    output_types: sign.outputs.clone(),
+                };
+
+                let shared_string = SharedString(sign.name);
+                let callable = Rc::new(Callable {
+                    it_instance: it_instance.clone(),
+                    it_module_func,
+                });
+
+                Ok((shared_string, callable))
             })
-            .map(|(adapter_function_type, export_function_names)| {
-                export_function_names
-                    .iter()
-                    .map(move |export_function_name| (*adapter_function_type, export_function_name))
-            })
-            .flatten()
-            .map(|(adapter_function_type, export_function_name)| {
-                let adapter_instructions = wit.adapter_by_type_r(adapter_function_type)?;
-                let wit_type = wit.type_by_idx_r(adapter_function_type)?;
+            .collect::<MResult<ExportFunctions>>()?;
 
-                match wit_type {
-                    ITAstType::Function {
-                        arguments,
-                        output_types,
-                    } => {
-                        let interpreter: ITInterpreter = adapter_instructions.clone().try_into()?;
-                        let wit_module_func = ITModuleFunc {
-                            interpreter: Arc::new(interpreter),
-                            arguments: arguments.clone(),
-                            output_types: output_types.clone(),
-                        };
-
-                        let shared_string = SharedString(Rc::new(export_function_name.to_string()));
-                        let callable = Rc::new(Callable {
-                            wit_instance: wit_instance.clone(),
-                            wit_module_func,
-                        });
-
-                        Ok((shared_string, callable))
-                    }
-                    _ => Err(MError::IncorrectWIT(format!(
-                        "type with idx = {} isn't a function type",
-                        adapter_function_type
-                    ))),
-                }
-            })
-            .collect::<MResult<ExportFunctions>>()
+        Ok((export_funcs, module_interface.export_record_types))
     }
 
     // this function deals only with import functions that have an adaptor implementation
@@ -442,92 +413,4 @@ impl MModule {
 
         Ok(import_object)
     }
-
-    // TODO : move it to a separate crate
-    fn extract_export_record_types(
-        export_funcs: &ExportFunctions,
-        wit_instance: &Arc<ITInstance>,
-    ) -> MResult<RecordTypes> {
-        use marine_it_generator::TYPE_RESOLVE_RECURSION_LIMIT;
-        use MError::RecordResolveError;
-
-        fn handle_itype(
-            itype: &IType,
-            wit_instance: &Arc<ITInstance>,
-            export_record_types: &mut RecordTypes,
-            recursion_level: u32,
-        ) -> MResult<()> {
-            use wasmer_it::interpreter::wasm::structures::Instance;
-
-            if recursion_level > TYPE_RESOLVE_RECURSION_LIMIT {
-                return Err(RecordResolveError(String::from(
-                    "mailformed module: a record contains more recursion level then allowed",
-                )));
-            }
-
-            fn handle_record_type(
-                record_type_id: u64,
-                wit_instance: &Arc<ITInstance>,
-                export_record_types: &mut RecordTypes,
-                recursion_level: u32,
-            ) -> MResult<()> {
-                let record_type =
-                    wit_instance
-                        .wit_record_by_id(record_type_id)
-                        .ok_or_else(|| {
-                            RecordResolveError(format!(
-                                "record type with type id {} not found",
-                                record_type_id
-                            ))
-                        })?;
-                export_record_types.insert(record_type_id, record_type.clone());
-
-                for field in record_type.fields.iter() {
-                    handle_itype(
-                        &field.ty,
-                        wit_instance,
-                        export_record_types,
-                        recursion_level + 1,
-                    )?;
-                }
-
-                Ok(())
-            }
-
-            match itype {
-                IType::Record(record_type_id) => handle_record_type(
-                    *record_type_id,
-                    wit_instance,
-                    export_record_types,
-                    recursion_level + 1,
-                )?,
-                IType::Array(array_ty) => handle_itype(
-                    array_ty,
-                    wit_instance,
-                    export_record_types,
-                    recursion_level + 1,
-                )?,
-                _ => {}
-            }
-
-            Ok(())
-        }
-
-        let mut export_record_types = HashMap::new();
-
-        let itypes = export_funcs.iter().flat_map(|(_, ref mut callable)| {
-            callable
-                .wit_module_func
-                .arguments
-                .iter()
-                .map(|arg| &arg.ty)
-                .chain(callable.wit_module_func.output_types.iter())
-        });
-
-        for itype in itypes {
-            handle_itype(itype, wit_instance, &mut export_record_types, 0)?;
-        }
-
-        Ok(export_record_types)
-    }
 }
